#!/usr/bin/env python3
"""The psychic who connect Docker world with XServer world.
Help you run GUI app with Docker. :^)

Usage:
    docker-psychic [MODE] [options] <image>
    docker-psychic -h|--help

Modes:
    cli                  For CLI software
    gui                  For GUI software

Options:
    -d, --debug          Output debug info
    -P, --persistent     Reserve container when you close application,
                         then you can use 'docker start' to launcher application.
    -p, --proxy          Enable system proxy support
    -a, --audio          Enable audio support
    -i, --im             Enable input method support


"""
from docopt import docopt
import os
import subprocess
import sys

__VERSION__ = '0.3.2'

def parse_host_env(env):
    """Abtain environment from Host machine."""
    result = os.getenv(env)
    if result:
        return ['-e', env+'='+os.getenv(env)]
    else:
        return []



def run_mode(mode):
    if mode == 'cli':
        return ['-it'] + parse_host_env('TERM')
    elif mode == 'gui':
        return ['-v', '/tmp/.X11-unix:/tmp/.X11-unix:ro'] +\
               ['-e', 'DISPLAY=unix'+os.getenv('DISPLAY')]


def persistent(flag):
    if flag:
        return []
    else:
        return ['--rm']


def proxy(flag):
    if flag:
        proxy_env_http_proxy = parse_host_env('http_proxy')
        proxy_env_https_proxy = parse_host_env('https_proxy')
        proxy_env_ftp_proxy = parse_host_env('ftp_proxy')
        proxy_env_rsync_proxy = parse_host_env('rsync_proxy')
        proxy = proxy_env_http_proxy + proxy_env_https_proxy +\
                proxy_env_ftp_proxy + proxy_env_rsync_proxy
    else:
        proxy = []

    return proxy


def audio(flag):
    if flag:
        return ['--device', '/dev/snd']
    else:
        return []


def im(flag):
    """Set input method"""
    if flag:
        im_env_lc_ctype = parse_host_env('LC_CTYPE')
        im_env_ximprogram = parse_host_env('XIMPROGRAM')
        im_env_xmodifiers = parse_host_env('XMODIFIERS')
        im_env_gtk_im_module = parse_host_env('GTK_IM_MODULE')
        im_env_qt_im_module = parse_host_env('QT_IM_MODULE')
        im = im_env_lc_ctype + im_env_ximprogram +\
             im_env_xmodifiers +\
             im_env_gtk_im_module + im_env_qt_im_module
    else:
        im = []

    return im


def link():
    """Used to exchange data between host and container."""
    host_end = os.path.join(os.path.expanduser('~'),
                            'docker-psychic')
    if not os.path.isdir(host_end):
        os.mkdir(host_end)
        uid = os.getuid()
        gid = os.getgid()
        os.chown(host_end, uid, gid)

    container_end = '/root/docker-pyschic'

    return [
        '-v', host_end+':'+container_end,
        '-w', container_end
    ]


def main():
    args = docopt(__doc__, version=__VERSION__)

    debug_flag=args['--debug']

    mode = args['MODE']

    audio_flag = args['--audio']
    persistent_flag = args['--persistent']
    proxy_flag = args['--proxy']
    im_flag = args['--im']
    image_name = args['<image>']
    container_name = image_name.split('/').pop()

    docker_cmd = ['docker', 'run'] +\
                 run_mode(mode) +\
                 persistent(persistent_flag) +\
                 ['-v', '/etc/localtime:/etc/localtime:ro'] +\
                 ['--net', 'host'] +\
                 proxy(proxy_flag) +\
                 audio(audio_flag) +\
                 im(im_flag) +\
                 link() +\
                 ['--name', container_name] +\
                 [image_name]

    xhost_cmd = "xhost + local:docker"

    if args['--debug']:
        print(args)

        proc_xhost = subprocess.Popen(xhost_cmd, shell=True)
        proc_xhost.wait()

        print(' '.join(docker_cmd))
        proc_docker = subprocess.Popen(docker_cmd)
        proc_docker.wait()  # wait docker process when in debug mode
    else:
        proc_xhost = subprocess.Popen(xhost_cmd, shell=True,
                                      stdout=subprocess.DEVNULL,
                                      stderr=subprocess.DEVNULL)
        proc_xhost.wait()

        if args['MODE'] == 'cli':
            proc_docker = subprocess.Popen(docker_cmd)
            proc_docker.wait()
        elif args['MODE'] == 'gui':
            proc_docker = subprocess.Popen(docker_cmd,
                                           stdout=subprocess.DEVNULL,
                                           stderr=subprocess.DEVNULL)
        # don't wait docker process when out of debug mode


if __name__ == '__main__':
    main()

## Another auth method
#XAUTH=/tmp/.docker.xauth
#xauth nlist :0 | sed -e 's/^..../ffff/' | xauth -f $XAUTH nmerge -
#-v $XAUTH:$XAUTH
#-e XAUTHORITY=$XAUTH
